"""Retrieves Steam User Stats using Steam Web API"""

import logging
import time
import requests
from api.utils import get_steam_credentials

# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)

# A reasonable timeout for the request (connection and read timeout)
REQUEST_TIMEOUT = (25, 30)
MAX_RETRIES = 5

# Steam Web API endpoints
PLAYER_SUMMARIES = "https://api.steampowered.com/ISteamUser/GetPlayerSummaries/v0002/"
RECENTLY_PLAYED_GAMES = (
    "https://api.steampowered.com/IPlayerService/GetRecentlyPlayedGames/v0001/"
)


def get_player_summaries():
    """Get Player Summaries from Steam
    Fetch basic profile information for a list of 64-bit Steam IDs"""
    creds = get_steam_credentials()
    steam_id = creds["steam_id"]
    api_key = creds["api_key"]

    url = f"{PLAYER_SUMMARIES}?key={api_key}&steamids={steam_id}"
    for _ in range(MAX_RETRIES):
        try:
            response = requests.get(url, timeout=REQUEST_TIMEOUT)
            if response.status_code == 429:
                retry_after = int(response.headers.get("Retry-After", 2))
                logger.warning("Rate limited. Sleeping for %s seconds", retry_after)
                time.sleep(retry_after)
                continue
            response.raise_for_status()
            return response.json()
        except requests.exceptions.HTTPError as err:
            logger.error("HTTP error: %s", err)
            return None
        except requests.exceptions.RequestException as err:
            logger.error("Request error: %s", err)
            return None
    logger.error("Max retries reached. Failed to fetch player summaries.")
    return None


def get_recently_played_games():
    """Fetch a list of games a player has played in the last two weeks"""
    creds = get_steam_credentials()
    steam_id = creds["steam_id"]
    api_key = creds["api_key"]

    url = f"{RECENTLY_PLAYED_GAMES}?key={api_key}&steamid={steam_id}&format=json"
    try:
        response = requests.get(url, timeout=REQUEST_TIMEOUT)
        response.raise_for_status()
        data = response.json()
        if data["response"].get("total_count", 0) == 0:
            return None
        return data
    except requests.exceptions.HTTPError as err:
        logger.error("HTTP error occurred: %s", err)
        return None
    except requests.exceptions.RequestException as err:
        logger.error("An error occurred: %s", err)
        return None



"""Generate Cards for Steam Stats"""

import datetime
import logging
import math
import os
import asyncio
from typing import Optional, TypedDict
from playwright.async_api import async_playwright, Error as PlaywrightError
from api.utils import get_asset_paths

# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


MARGIN = 10
CARD_SELECTOR = ".card"


class FloatRect(TypedDict):
    """Define FloatRect type for compatibility with Playwright"""

    x: float
    y: float
    width: float
    height: float


# Get GitHub Repo's Details where the action is being run
repo_owner, repo_name = os.getenv("GITHUB_REPOSITORY", "owner/repo").split("/")
branch_name = os.getenv("GITHUB_REF_NAME", "main")

# Persona state mapping for Steam Profile Status
personastate_map = {
    0: "Offline",
    1: "Online",
    2: "Busy",
    3: "Away",
    4: "Snooze",
    5: "Looking to trade",
    6: "Looking to play",
}


def handle_exception(e):
    """Handle exceptions and log appropriate error messages"""
    if isinstance(e, FileNotFoundError):
        logger.error("File Not Found Error: %s", e)
    elif isinstance(e, PlaywrightError):
        logger.error("Playwright Error: %s", e)
    elif isinstance(e, KeyError):
        logger.error("Key Error: %s", e)
    elif isinstance(e, asyncio.TimeoutError):
        logger.error("Timeout Error: %s", e)
    elif isinstance(e, ValueError):
        logger.error("Value Error: %s", e)
    else:
        logger.error("Unexpected Error: %s", e)


async def get_element_bounding_box(
    html_file: str, selector: str, margin: int = MARGIN
) -> Optional[FloatRect]:
    """Get the bounding box of the specified element using Playwright"""
    browser = None
    try:
        # Check if the HTML file exists
        if not os.path.exists(html_file):
            raise FileNotFoundError("HTML file not found: " + html_file)

        async with async_playwright() as p:
            browser = await p.chromium.launch(headless=True)
            page = await browser.new_page()
            try:
                await page.goto("file://" + os.path.abspath(html_file))
            except TimeoutError as e:
                raise asyncio.TimeoutError("Timeout error while loading page") from e

            # Find element and get its bounding box
            element = await page.query_selector(selector)
            if not element:
                raise ValueError("Element not found for selector: " + selector)

            try:
                bounding_box = await element.bounding_box()
            except KeyError as e:
                raise KeyError("Key error while retrieving bounding box") from e

            if not bounding_box:
                raise ValueError(
                    "Could not retrieve bounding box for selector: " + selector
                )

            # Add margin to the bounding box
            bounding_box_with_margin: FloatRect = {
                "x": max(bounding_box["x"] - margin, 0),
                "y": max(bounding_box["y"] - margin, 0),
                "width": bounding_box["width"] + 2 * margin,
                "height": bounding_box["height"] + 2 * margin,
            }

            await browser.close()
            return bounding_box_with_margin

    except (
        FileNotFoundError,
        PlaywrightError,
        ValueError,
        KeyError,
        asyncio.TimeoutError,
    ) as e:
        handle_exception(e)
        return None
    finally:
        if browser:
            await browser.close()


async def html_to_png(
    html_file: str, output_file: str, selector: str, margin: int = MARGIN
) -> bool:
    """Convert an HTML file to a PNG using Playwright with clipping"""
    bounding_box = await get_element_bounding_box(html_file, selector, margin)
    if not bounding_box:
        logger.error("Bounding box could not be determined")
        return False

    clip: FloatRect = {
        "x": float(bounding_box["x"]),
        "y": float(bounding_box["y"]),
        "width": float(bounding_box["width"]),
        "height": float(bounding_box["height"]),
    }

    browser = None
    try:
        async with async_playwright() as p:
            browser = await p.chromium.launch(headless=True)
            page = await browser.new_page()
            await page.goto("file://" + os.path.abspath(html_file))

            # Take screenshot with clipping
            await page.screenshot(path=output_file, clip=clip)
            return True

    except (PlaywrightError, asyncio.TimeoutError) as e:
        handle_exception(e)
        return False

    finally:
        if browser:
            await browser.close()


def convert_html_to_png(html_file, output_file, selector):
    """Synchronous wrapper to convert HTML to PNG"""
    try:
        return asyncio.run(html_to_png(html_file, output_file, selector))
    except (FileNotFoundError, PlaywrightError, KeyError, asyncio.TimeoutError) as e:
        handle_exception(e)
        return False


def format_unix_time(unix_time):
    """Convert Unix time to human-readable format with ordinal day"""
    dt = datetime.datetime.fromtimestamp(unix_time)
    day = dt.day
    suffix = (
        "th" if 11 <= day <= 13 else {1: "st", 2: "nd", 3: "rd"}.get(day % 10, "th")
    )
    return f"{day}{suffix} {dt.strftime('%b %Y')}"


def generate_card_for_player_summary(player_data):
    """Generate HTML content based on Steam Player Summary Data"""
    if not player_data:
        return None

    summary = player_data["response"]["players"][0]
    player = {
        "name": summary.get("personaname", "Unknown"),
        "status": personastate_map.get(summary.get("personastate", 0), "Unknown"),
        "avatar": summary.get("avatarfull", ""),
        "country": summary.get("loccountrycode", ""),
        "lastlogoff": format_unix_time(summary.get("lastlogoff", 0)),
        "timecreated": format_unix_time(summary.get("timecreated", 0)),
        "game": summary.get("gameextrainfo"),
    }

    country_section = (
        f"""
        <p id="country">Country: <span id="country-code">{player['country']}</span>
            <img id="flag" class="flag"
            src="https://flagcdn.com/w320/{player['country'].lower()}.png" alt="Flag">
        </p>
        """
        if player["country"]
        else ""
    )

    game_section = (
        f"<p id='game'>Currently Playing: <span id='game-info'>{player['game']}</span></p>"
        if player["game"]
        else ""
    )

    html_content = f"""
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>Steam Profile Summary</title>
        <style>
            .card {{
                width: 100%;
                max-width: 400px;
                margin: auto;
                border: 2px solid #000;
                padding: 15px;
                box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
                border-radius: 10px;
                background-color: #fff;
            }}
            .avatar {{
                width: 80px;
                height: 80px;
                border-radius: 50%;
                margin: auto;
            }}
            .content {{
                text-align: center;
            }}
            .flag {{
                width: 32px;
                height: 24px;
                vertical-align: middle;
            }}
            .info-container {{
                display: flex;
                justify-content: space-between;
                margin-top: 10px;
            }}
            .info-left, .info-right {{
                width: 48%;
            }}
        </style>
    </head>
    <body>
        <div class="card">
            <div class="content">
                <h2>Steam Profile Summary</h2>
                <img id="avatar" class="avatar" src="{player['avatar']}" alt="Avatar">
                <h2 id="name">{player['name']}</h2>
                <div class="info-container">
                    <div class="info-left">
                        <p id="status">Status: {player['status']}</p>
                        {country_section}
                    </div>
                    <div class="info-right">
                        <p id="lastlogoff">Last Logoff: {player['lastlogoff']}</p>
                        <p id="timecreated">PC Gaming Since: {player['timecreated']}</p>
                    </div>
                </div>
                {game_section}
            </div>
        </div>
    </body>
    </html>
    """

    html_path, png_path, relative_png_path = get_asset_paths("steam_summary")
    with open(html_path, "w", encoding="utf-8") as file:
        file.write(html_content)
    convert_html_to_png(html_path, png_path, CARD_SELECTOR)

    return (
        f"![Steam Summary](https://github.com/{repo_owner}/"
        f"{repo_name}/blob/{branch_name}/{relative_png_path})\n"
    )


def format_playtime(playtime):
    """Format playtime into human-readable format"""
    if playtime < 60:
        unit = "min" if playtime == 1 else "mins"
        return f"{playtime} {unit}"
    hours, minutes = divmod(playtime, 60)
    if minutes == 0:
        return f"{hours} hrs"
    unit = "min" if minutes == 1 else "mins"
    return f"{hours} hrs and {minutes} {unit}"


def generate_progress_bar(game, index, max_playtime, log_scale, placeholder_image):
    """Generate progress bar HTML for a single game"""
    name = game.get("name", "Unknown Game")
    playtime = game.get("playtime_2weeks", 0)
    img_icon_url = (
        placeholder_image
        if name == "Spacewar"
        else (
            f"https://media.steampowered.com/steamcommunity/public/images/apps/"
            f"{game.get('appid')}/{game.get('img_icon_url')}.jpg"
        )
    )
    normalized_playtime = (
        round(math.log1p(playtime) / math.log1p(max_playtime) * 100)
        if log_scale
        else round((playtime / max_playtime) * 100)
    )
    display_time = format_playtime(playtime)
    style_class = f"progress-style-{(index % 6) + 1}"

    return f"""
    <div class="bar-container">
        <img src="{img_icon_url}" alt="{name}" class="game-icon">
        <progress class="{style_class}" value="{normalized_playtime}" max="100"></progress>
        <div class="game-info">
            <span class="game-name">{name}</span><br>
            <span class="game-time">{display_time}</span>
        </div>
    </div>
    """


def generate_card_for_played_games(games_data):
    """Generate HTML Card for recently played games in last 2 weeks"""
    if not games_data:
        return None

    placeholder_image = "https://i.imgur.com/DBnVqet.jpg"
    log_scale = os.getenv("INPUT_LOG_SCALE", "false").lower() in ("true", "1", "t")
    watermark = '<div class="watermark">Log Scale Enabled</div>' if log_scale else ""

    num_games = len(games_data["response"]["games"])
    min_canvas_height = num_games * 60 + 70
    max_playtime = (
        max(game["playtime_2weeks"] for game in games_data["response"]["games"]) or 1
    )

    progress_bars = "".join(
        generate_progress_bar(game, i, max_playtime, log_scale, placeholder_image)
        for i, game in enumerate(games_data["response"]["games"])
        if "name" in game and "playtime_2weeks" in game
    )

    html_content = f"""
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>Recently Played Games in Last 2 Weeks</title>
        <link rel="stylesheet" href="style.css">
    </head>
    <body>
        <div class="card" style="height: {min_canvas_height}px; position: relative; ">
            <div class="content" style="position: relative; text-align: center;">
                <h2>Recently Played Games (Last 2 Weeks)</h2>
                {progress_bars}
            </div>
            {watermark}
        </div>
    </body>
    </html>
    """

    html_path, png_path, relative_png_path = get_asset_paths("recently_played_games")
    with open(html_path, "w", encoding="utf-8") as file:
        file.write(html_content)
    convert_html_to_png(html_path, png_path, CARD_SELECTOR)

    return (
        f"![Recently Played Games](https://github.com/{repo_owner}/"
        f"{repo_name}/blob/{branch_name}/{relative_png_path})"
    )


def generate_card_for_steam_workshop(workshop_stats):
    """Generates HTML content for retrieved Workshop Data"""
    html_content = f"""
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>Steam Workshop Stats</title>
        <style>
            body {{
                font-family: Arial, sans-serif;
                display: flex;
                justify-content: center;
                align-items: center;
                min-height: 100vh;
                background-color: #f0f0f0;
                margin: 0;
            }}
            .card {{
                width: 100%;
                max-width: 400px;
                margin: auto;
                border: 2px solid #000;
                padding: 20px;
                box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
                border-radius: 10px;
                background-color: #fff;
                text-align: center;
            }}
            table {{
                width: 100%;
                border-collapse: collapse;
            }}
            th, td {{
                border: 1px solid #ddd;
                padding: 8px;
                text-align: center;
            }}
            th {{
                background-color: #6495ED;
                color: white;
            }}
            tr:nth-child(even) {{
                background-color: #f2f2f2;
            }}
        </style>
    </head>
    <body>
        <div class="card">
            <h2>Steam Workshop Stats</h2>
            <table>
                <thead>
                    <tr>
                        <th>Workshop Stats</th>
                        <th>Total</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td>Unique Visitors</td>
                        <td>{workshop_stats.get("total_unique_visitors", 0)}</td>
                    </tr>
                    <tr>
                        <td>Current Subscribers</td>
                        <td>{workshop_stats.get("total_current_subscribers", 0)}</td>
                    </tr>
                    <tr>
                        <td>Current Favorites</td>
                        <td>{workshop_stats.get("total_current_favorites", 0)}</td>
                    </tr>
                </tbody>
            </table>
        </div>
    </body>
    </html>
    """

    html_path, png_path, relative_png_path = get_asset_paths("steam_workshop_stats")
    with open(html_path, "w", encoding="utf-8") as file:
        file.write(html_content)
    convert_html_to_png(html_path, png_path, CARD_SELECTOR)

    return (
        f"![Steam Workshop Stats](https://github.com/{repo_owner}/"
        f"{repo_name}/blob/{branch_name}/{relative_png_path})"
    )



"""Main Runner Script"""

import logging
import os
import time
from api.steam_stats import get_player_summaries, get_recently_played_games
from api.steam_workshop import fetch_workshop_item_links, fetch_all_workshop_stats
from api.card import (
    generate_card_for_player_summary,
    generate_card_for_played_games,
    generate_card_for_steam_workshop,
)
from api.utils import (
    get_steam_credentials,
    initialize_github,
    create_tree_elements,
    get_readme_content,
    get_asset_paths,
    README,
)

# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)

# Optional Feature Flag
WORKSHOP_STATS = os.getenv("INPUT_WORKSHOP_STATS", "false").lower() in (
    "true",
    "1",
    "t",
)

# Version Identifier for Changelog
__version__ = "1.3.1"


def update_readme(repo, markdown_data, start_marker, end_marker):
    """Updates the README.md file with the provided Markdown content within specified markers"""
    try:
        readme_file = repo.get_contents(README)
        readme_content = readme_file.decoded_content.decode("utf-8")

        start_index = readme_content.find(start_marker)
        end_index = readme_content.find(end_marker, start_index)

        if start_index == -1 or end_index == -1:
            logger.error("Markers not found: %s", start_marker)
            return None

        new_section_content = start_marker + "\n" + markdown_data + "\n" + end_marker

        if (
            new_section_content
            != readme_content[start_index : end_index + len(end_marker)]
        ):
            return new_section_content

        return None

    except (FileNotFoundError, PermissionError, IOError) as e:
        logger.error("Error occurred while updating README: %s", str(e))
        return None


def generate_steam_stats():
    """Generate Steam Stats and return markdown content"""
    user_markdown_content = ""
    player_summary = get_player_summaries()
    if player_summary:
        logger.info("Retrieved Steam User Data")
        summary_content = generate_card_for_player_summary(player_summary)
        if summary_content:
            user_markdown_content += summary_content
            logger.info("Generated Card for Steam User Data")
        else:
            logger.error("Failed to generate card for Steam Summary")
    else:
        logger.info("No Steam User Summary data found")

    recently_played_games = get_recently_played_games()
    if recently_played_games:
        logger.info("Retrieved Recently Played Games Data")
        recent_games = generate_card_for_played_games(recently_played_games)
        if recent_games:
            user_markdown_content += recent_games
            logger.info("Generated Card for Recently Played Games")
        else:
            logger.info("No Games data found, skipping card generation")
    else:
        logger.info("No Recently Played Games data found")

    return user_markdown_content


def generate_workshop_stats():
    """Generate Workshop Stats and return markdown content"""
    creds = get_steam_credentials()
    custom_id = creds["custom_id"]
    api_key = creds["api_key"]
    workshop_markdown_content = ""
    links = fetch_workshop_item_links(custom_id, api_key)
    if links:
        workshop_data = fetch_all_workshop_stats(links)
        workshop_content = generate_card_for_steam_workshop(workshop_data)
        if workshop_content:
            workshop_markdown_content += workshop_content
            logger.info("Generated Card for Workshop Stats")
        else:
            logger.error("Failed to generate card data for Workshop Stats")
    else:
        logger.error("No workshop content was found")

    return workshop_markdown_content


def commit_to_github(repo, files_to_update):
    """Commit files to GitHub Repo"""
    if not files_to_update:
        logger.info("No changes to commit")
        return True

    try:
        branch = repo.get_branch(repo.default_branch)
        last_commit_sha = branch.commit.sha

        tree_elements = create_tree_elements(repo, files_to_update)
        new_tree = repo.create_git_tree(
            tree_elements, repo.get_git_tree(last_commit_sha)
        )

        new_commit = repo.create_git_commit(
            message="chore: Update Steam Stats",
            tree=new_tree,
            parents=[repo.get_git_commit(last_commit_sha)],
        )

        ref = repo.get_git_ref("heads/" + branch.name)
        ref.edit(sha=new_commit.sha)
        return True

    except (ValueError, IOError) as e:
        logger.error("Error occurred while committing to GitHub: %s", str(e))
        return False


def update_readme_sections(repo, current_content):
    """Update README sections with Steam and Workshop stats"""
    updated_content = current_content

    steam_stats_content = generate_steam_stats()
    if steam_stats_content:
        updated_content = update_section(
            repo,
            updated_content,
            steam_stats_content,
            "<!-- Steam-Stats start -->",
            "<!-- Steam-Stats end -->",
        )
        logger.info("Steam stats updated")
    else:
        logger.info("No Steam stats content generated")

    if WORKSHOP_STATS:
        workshop_content = generate_workshop_stats()
        if workshop_content:
            updated_content = update_section(
                repo,
                updated_content,
                workshop_content,
                "<!-- Steam-Workshop start -->",
                "<!-- Steam-Workshop end -->",
            )
            logger.info("Workshop stats updated")
        else:
            logger.info("No Workshop stats content generated")

    return updated_content


def update_section(repo, current_content, new_content, start_marker, end_marker):
    """Update a section of the README using markers"""
    updated_section = update_readme(repo, new_content, start_marker, end_marker)
    if updated_section:
        start_index = current_content.find(start_marker)
        end_index = current_content.find(end_marker, start_index) + len(end_marker)
        return (
            current_content[:start_index]
            + updated_section
            + current_content[end_index:]
        )
    return current_content


def collect_files_to_update(current_readme, original_readme):
    """Collect files that needs to be updated"""
    files_to_update = {}
    if current_readme != original_readme:
        files_to_update[README] = current_readme.encode("utf-8")

    for name in [
        "steam_summary",
        "recently_played_games",
        "steam_workshop_stats",
    ]:
        _, png_path, relative_png_path = get_asset_paths(name)
        if png_path.exists():
            with open(png_path, "rb") as file:
                files_to_update[relative_png_path] = file.read()

    return files_to_update


def log_execution_time(start_time):
    """Log the total execution time"""
    total_time = round(time.time() - start_time, 3)
    if total_time > 60:
        minutes, seconds = divmod(total_time, 60)
        logger.info(
            "Total Execution Time: %d minutes and %.3f seconds", minutes, seconds
        )
    else:
        logger.info("Total Execution Time: %.3f seconds", total_time)


def main():
    """Main function to run the script"""
    start_time = time.time()

    try:
        repo = initialize_github()
        original_readme = get_readme_content(repo)
        current_readme = update_readme_sections(repo, original_readme)
        files_to_update = collect_files_to_update(current_readme, original_readme)

        if files_to_update:
            if commit_to_github(repo, files_to_update):
                logger.info("Successfully committed to GitHub")
            else:
                logger.error("Failed to commit changes to GitHub")
        else:
            logger.info("No changes to commit")

    except (ValueError, IOError) as e:
        logger.error("%s error occurred: %s", type(e).__name__, str(e))

    log_execution_time(start_time)


if __name__ == "__main__":
    main()



"""Utility functions for the API"""

import base64
import logging
import os
from pathlib import Path
from github import Github, InputGitTreeElement, Auth

REPO_ROOT = Path(__file__).resolve().parent.parent
ASSETS_DIR = REPO_ROOT / "assets"
README = "README.md"

# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


def get_steam_credentials():
    """Fetch Steam credentials using environment variables"""
    return {
        "steam_id": os.environ["INPUT_STEAM_ID"],
        "api_key": os.environ["INPUT_STEAM_API_KEY"],
        "custom_id": os.environ["INPUT_STEAM_CUSTOM_ID"],
    }


def get_github_token():
    """Get GitHub token from environment variables"""
    token = (
        os.environ.get("GITHUB_TOKEN")
        or os.environ.get("GH_TOKEN")
        or os.environ.get("INPUT_GH_TOKEN")
    )
    if not token:
        raise ValueError(
            "No GitHub token found in env vars (GITHUB_TOKEN, GH_TOKEN or INPUT_GH_TOKEN"
        )
    return token


def get_repo(g):
    """Get Repo object"""
    repo_name = os.environ.get("GITHUB_REPOSITORY")
    if not repo_name:
        raise ValueError("GITHUB_REPOSITORY environment variable not set")
    return g.get_repo(repo_name)


def initialize_github():
    """Initialize GitHub client and get repo"""
    token = get_github_token()
    g = Github(auth=Auth.Token(token))
    return get_repo(g)


def get_readme_content(repo):
    """Get current README content"""
    readme_file = repo.get_contents(README)
    if isinstance(readme_file, list):
        readme_file = readme_file[0]
    return readme_file.decoded_content.decode("utf-8")


def create_tree_elements(repo, files_to_update):
    """Create tree elements for GitHub commit"""
    tree_elements = []
    for file_path, file_content in files_to_update.items():
        if isinstance(file_content, bytes):
            content = base64.b64encode(file_content).decode("utf-8")
            encoding = "base64"
        else:
            content = file_content
            encoding = "utf-8"

        logger.info("Committing file: %s", file_path)
        blob = repo.create_git_blob(content, encoding)
        element = InputGitTreeElement(
            path=file_path, mode="100644", type="blob", sha=blob.sha
        )
        tree_elements.append(element)
    return tree_elements


def get_asset_paths(filename_base: str) -> tuple[Path, Path, str]:
    """Returns the absolute HTML, PNG & relative PNG path for embedding in markdown"""
    html_path = ASSETS_DIR / f"{filename_base}.html"
    png_path = ASSETS_DIR / f"{filename_base}.png"
    relative_png_path = png_path.relative_to(REPO_ROOT).as_posix()
    return html_path, png_path, relative_png_path



"""Scrape Steam Workshop Data"""

import logging
import requests
from bs4 import BeautifulSoup, Tag

# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)

# A reasonable timeout for the request (connection and read timeout)
REQUEST_TIMEOUT = (25, 30)

GET_SERVER_INFO_URL = "https://api.steampowered.com/ISteamWebAPIUtil/GetServerInfo/v1/"


def get_server_info(api_key):
    """Fetch server information from the Steam Web API"""
    try:
        response = requests.get(
            GET_SERVER_INFO_URL, params={"key": api_key}, timeout=REQUEST_TIMEOUT
        )
        response.raise_for_status()
        return response.json()
    except requests.exceptions.RequestException as e:
        logger.error("Error fetching server info: %s", str(e))
        return None


def is_server_online(api_key):
    """Check if the server is online based on the GetServerInfo response"""
    serverinfo = get_server_info(api_key)
    if serverinfo and "servertime" in serverinfo:
        return True
    return False


def extract_links(workshop_items):
    """Extract links from workshop items"""
    links = []
    for item in workshop_items:
        link_tag = item.find("a", class_="ugc")
        if link_tag and "href" in link_tag.attrs:
            links.append(link_tag["href"])
    return links


def has_next_page(soup):
    """Check if there is a next page"""
    paging_controls = soup.find("div", class_="workshopBrowsePagingControls")
    if paging_controls:
        next_page_link = paging_controls.find("a", class_="pagebtn", string=">")
        return next_page_link is not None
    return False


def handle_request_exception(e):
    """Handle request exceptions"""
    if isinstance(e, requests.exceptions.ConnectionError):
        logger.error("Connection error occurred. Please check your network connection")
    elif isinstance(e, requests.exceptions.Timeout):
        logger.error("Request timed out. Please try again later")
    elif isinstance(e, requests.exceptions.TooManyRedirects):
        logger.error("Too many redirects. Check the URL and try again")
    elif isinstance(e, requests.exceptions.HTTPError):
        logger.error(
            "HTTP error occurred: %s - %s",
            str(e.response.status_code),
            str(e.response.reason),
        )
    else:
        logger.error("An error occurred: %s", str(e))


def fetch_workshop_item_links(steam_id, api_key):
    """Fetch each workshop item's link, navigating through all pages"""
    if not is_server_online(api_key):
        raise ConnectionError(
            "Steam Community is currently offline. Please try again later"
        )

    base_url = "https://steamcommunity.com/id/" + str(steam_id) + "/myworkshopfiles/"
    item_links = []
    page_number = 1

    while True:
        url = str(base_url) + "?p=" + str(page_number)
        try:
            response = requests.get(url, timeout=REQUEST_TIMEOUT)
            response.raise_for_status()
            soup = BeautifulSoup(response.content, "html.parser")
            workshop_items = soup.find_all("div", class_="workshopItem")

            if not workshop_items:
                logger.info("No workshop items found on page %d", page_number)
                break

            item_links.extend(extract_links(workshop_items))

            if not has_next_page(soup):
                break

            page_number += 1

        except requests.exceptions.RequestException as e:
            handle_request_exception(e)
            break
        except AttributeError as e:
            logger.error(
                "Parsing error: %s. The structure of the page might have changed",
                str(e),
            )
            break

    if not item_links and page_number == 1:
        raise ValueError("No items were found in your Steam Workshop")

    return item_links


def parse_stats_table(stats_table):
    """Parse the stats table and return a dictionary of stats"""
    stats = {}
    if not isinstance(stats_table, Tag):
        return stats
    for row in stats_table.find_all("tr"):
        cells = row.find_all("td")
        if len(cells) == 2:
            key = cells[1].text.strip().lower().replace(" ", "_")
            value = cells[0].text.strip().replace(",", "")
            try:
                stats[key] = int(value) if value else 0
            except ValueError:
                logger.error("Could not convert value to int: %s", str(value))
                stats[key] = 0
    return stats


def fetch_individual_workshop_stats(item_url):
    """Fetch Author Stats from a Workshop Item"""
    stats = {}

    try:
        response = requests.get(item_url, timeout=REQUEST_TIMEOUT)
        response.raise_for_status()
        soup = BeautifulSoup(response.content, "html.parser")
        stats_table = soup.find("table", class_="stats_table")

        if isinstance(stats_table, Tag):
            stats = parse_stats_table(stats_table)
        else:
            logger.info("Could not find stats table at %s", str(item_url))
    except (requests.exceptions.RequestException, AttributeError) as e:
        handle_request_exception(e)
        return {}

    # Ensure all expected stats are present, even if they are zero
    stats["unique_visitors"] = stats.get("unique_visitors", 0)
    stats["current_subscribers"] = stats.get("current_subscribers", 0)
    stats["current_favorites"] = stats.get("current_favorites", 0)

    return stats


def fetch_all_workshop_stats(item_links):
    """Fetch Stats for all of the items in Steam User's Workshop"""
    all_stats = []

    for link in item_links:
        try:
            stats = fetch_individual_workshop_stats(link)
            if stats:
                all_stats.append(stats)
        except (requests.exceptions.RequestException, AttributeError) as e:
            handle_request_exception(e)

    total_unique_visitors = sum(item.get("unique_visitors", 0) for item in all_stats)
    total_current_subscribers = sum(
        item.get("current_subscribers", 0) for item in all_stats
    )
    total_current_favorites = sum(
        item.get("current_favorites", 0) for item in all_stats
    )

    return {
        "total_unique_visitors": total_unique_visitors,
        "total_current_subscribers": total_current_subscribers,
        "total_current_favorites": total_current_favorites,
        "individual_stats": all_stats,
    }
